clear matrix
clear mata
clear      
set more off, perm
capture log close
*=========================================================;
* Set working directory to the folder containing this file
cd /your/working/directory
*=========================================================;
# delimit ;
*=================================================================================================;
*=================================================================================================;
*=================================================================================================;
* File-Name:       	session4.do
* Date:            	17/07/16
* Author:          	DD	
* Machine:         	DD's MacBook Pro
*=================================================================================================;
*=================================================================================================;
* Programs and locals
*=================================================================================================;
* define a bunch of helpful locals;
local graphr "graphr(fc(white) lc(white) ifc(white) ilc(white))";
local graphrBlack "graphr(fc(white) lc(black) ifc(white) ilc(white))";
local grcom "ycommon xcommon imargin(small)";
*=================================================================================================;
*=================================================================================================;
*=================================================================================================;
* Session 4 
*=================================================================================================;
* Recap simulations and bootstrapping
*=================================================================================================;
* Resampling
*=================================================================================================;
* Bootstrap
*=========================================================;
program drop _all;
program plotHistAndQnorm;
	preserve;
	use boot, clear;
	sum theta;
	local mean = r(mean);
	local sd = r(sd);
	hist theta, graphr(fc(white) lc(white) ifc(white) ilc(white)) col(black) 
	xti(theta_hat) name(hist, replace) 
	plot(function normal = normalden(x,`mean',`sd'), ra(theta) lc(red)) leg(off)
	xline(`mean', lc(gray)) ti(Distribution of theta_hat, col(black));
	qnorm theta, graphr(fc(white) lc(white) ifc(white) ilc(white)) name(qnorm, replace) yti(theta_hat, col(black)) ysc(alt) mc(black) lc(red);
	gr combine hist qnorm, `graphr'; 
	restore;
end;

program drop _all;
program assessBias;
	args varname statistic command dataPath outputPath;
	tempname assessBias;
	postfile `assessBias' bias B smallOrLarge using assessBiasResults, replace;
	local k = 1;
	foreach dataSet in "" "Large" {;
		use `dataPath'/fakeData`dataSet', clear;
		sum `varname';
		local range = r(max) - r(min);
		local j = 1;
		foreach i in 10 20 50 100 200 500 1000 2000 5000 10000 {;
			`command';
			local theta = `statistic';
			bootstrap thetaHat = `statistic', seed(010101) reps(`i') nodots saving(boot, replace): `command';
			preserve;
			use boot, clear; 
			sum thetaHat;
			post `assessBias' (100*(r(mean) - `theta')/`range') (`i') (`k');
			local j = `j' + 1;
			restore;
		};
		local k = `k' + 1;
	};
	postclose `assessBias';
end;
	
*===============================================;
* t-test;
use fakeData, clear;
ttest var, by(cat);
bootstrap theta = (r(mu_1) - r(mu_2)), seed(010101) nodots saving(boot, replace): ttest var, by(cat);
plotHistAndQnorm;
gr export histAndQnormBootDiff.pdf, replace;

foreach dataSet in "" "Large" {;
	if("`dataSet'"=="Large"){;
		local label "l1(Large)";
	};
	if("`dataSet'"=="") {;
		local label "l1(Small)";
	};
	use fakeData`dataSet', clear;
	foreach i in 10 50 500 5000 {;
		if(`i'==10){;
			local ysc "";
		};
		else {;
			local ysc "off";
			local label "";
		};
		bootstrap theta = (r(mu_1) - r(mu_2)), seed(010101) reps(`i') nodots saving(boot, replace): 
		ttest var, by(cat);
		preserve;
		use boot, clear;
		sum theta;
		local mean = r(mean);
		local sd = r(sd);
		hist theta, graphr(fc(white) lc(white) ifc(white) ilc(white)) col(black) 
		xti(theta_hat) xline(`mean', lc(gray)) `graphr' col(black) xti(theta) ti(`i', col(black)) 
		plot(function normal = normalden(x,`mean',`sd'), ra(theta) lc(red)) 
		name(p`dataSet'`i', replace) ysc(`ysc') `label' leg(off);
		restore;
	};
};

gr combine p10 p50 p500 p5000 pLarge10 pLarge50 pLarge500 pLarge5000, rows(2) cols(4) `graphr'; 
gr export histBootDiff_byRep.pdf, replace; 

*===============================================;
* mean;
use fakeData, clear;
bootstrap theta = r(mean), seed(010101) nodots reps(3000) saving(boot, replace): sum var;
plotHistAndQnorm;
gr export histAndQnormBootMean.pdf, replace;

foreach dataSet in "" "Large" {;
	if("`dataSet'"=="Large"){;
		local label "l1(Large)";
	};
	if("`dataSet'"=="") {;
		local label "l1(Small)";
	};
	use fakeData`dataSet', clear;
	foreach i in 10 50 500 5000 {;
		if(`i'==10){;
			local ysc "";
		};
		else {;
			local ysc "off";
			local label "";
		};
		bootstrap theta = r(mean), seed(010101) reps(`i') nodots saving(boot, replace): sum var;
		preserve;
		use boot, clear;
		sum theta;
		local mean = r(mean);
		local sd = r(sd);
		hist theta, graphr(fc(white) lc(white) ifc(white) ilc(white)) col(black) 
		xti(theta_hat) xline(`mean', lc(gray)) `graphr' col(black) xti(theta) ti(`i', col(black)) 
		plot(function normal = normalden(x,`mean',`sd'), ra(theta) lc(red)) 
		name(p`dataSet'`i', replace) ysc(`ysc') `label' leg(off);
		restore;
	};
};

gr combine p10 p50 p500 p5000 pLarge10 pLarge50 pLarge500 pLarge5000, rows(2) cols(4) `graphr'; 
gr export histBootMean_byRep.pdf, replace; 

*===============================================;
* median;
use fakeData, clear;
bootstrap theta = r(p50), seed(010101) nodots reps(3000) saving(boot, replace): sum var, d;
plotHistAndQnorm;
gr export histAndQnormBootMedian.pdf, replace;

use fakeDataLarge, clear; 
bootstrap theta = r(p50), seed(010101) nodots reps(3000) saving(boot, replace): sum var, d;
plotHistAndQnorm;
gr export histAndQnormBootMedian_largeSample.pdf, replace;

foreach dataSet in "" "Large" {;
	if("`dataSet'"=="Large"){;
		local label "l1(Large)";
	};
	if("`dataSet'"=="") {;
		local label "l1(Small)";
	};
	use fakeData`dataSet', clear;
	foreach i in 3000 10000 100000 {;
		if(`i'==3000){;
			local ysc "";
		};
		else {;
			local ysc "off";
			local label "";
		};
		bootstrap theta = r(p50), seed(010101) reps(`i') nodots saving(boot, replace): 
		sum var, d;
		preserve;
		use boot, clear;
		sum theta;
		local mean = r(mean);
		local sd = r(sd);
		hist theta, graphr(fc(white) lc(white) ifc(white) ilc(white)) col(black) 
		xti(theta_hat) xline(`mean', lc(gray)) `graphr' col(black) xti(theta) ti(`i', col(black)) 
		plot(function normal = normalden(x,`mean',`sd'), ra(theta) lc(red)) 
		name(p`dataSet'`i', replace) ysc(`ysc') `label' leg(off);
		restore;
	};
};

gr combine p3000 p10000 p100000 pLarge3000 pLarge10000 pLarge100000, rows(2) cols(3) `graphr'; 
gr export histBootMedian_byRep.pdf, replace; 

*===============================================;
* assess bias;

* bootstrap mean;
assessBias var r(mean) "sum var" `data' `output'; 
use assessBiasResults, clear; 

gr tw 
	(connected bias B if(smallOrLarge==1), sort lc(red) mc(red))
	(connected bias B if(smallOrLarge==2), sort lc(blue) mc(blue)),
`graphr' leg(order(1 "Small" 2 "Large")) yti(Bias in percentage of variable range); 
gr export connectedBootBiasMean_byRep.pdf, replace; 

* bootstrap median;
assessBias var r(p50) "sum var, d" `data' `output'; 
use assessBiasResults, clear; 

gr tw 
	(connected bias B if(smallOrLarge==1), sort lc(red) mc(red))
	(connected bias B if(smallOrLarge==2), sort lc(blue) mc(blue)),
`graphr' leg(order(1 "Small" 2 "Large")); 
gr export connectedBootBiasMedian_byRep.pdf, replace; 

use fakeData, clear;
bootstrap theta = r(mean), seed(010101) nodots reps(3000) bca: sum var;
estat bootstrap, bca; 

bootstrap theta = r(mean), seed(010101) nodots reps(3000) bca: sum varExp;
estat bootstrap, bca; 

bootstrap theta = r(mean), seed(010101) nodots reps(3000) bca ties: sum var;
estat bootstrap, bca; 

*===============================================;
* assess skewness;
foreach dataSet in "" "Large" {;
	if("`dataSet'"=="Large"){;
		local label "l1(Large)";
	};
	if("`dataSet'"=="") {;
		local label "l1(Small)";
	};
	use fakeData`dataSet', clear;
	foreach i in 10 50 500 5000 {;
		if(`i'==10){;
			local ysc "";
		};
		else {;
			local ysc "off";
			local label "";
		};
		bootstrap theta = r(mean), seed(010101) reps(`i') nodots saving(boot, replace): sum varExp;
		preserve;
		use boot, clear;
		sum theta;
		local mean = r(mean);
		local sd = r(sd);
		hist theta, graphr(fc(white) lc(white) ifc(white) ilc(white)) col(black) 
		xti(theta_hat) xline(`mean', lc(gray)) col(black) xti(theta) ti(`i', col(black)) 
		plot(function normal = normalden(x,`mean',`sd'), ra(theta) lc(red)) 
		name(p`dataSet'`i', replace) ysc(`ysc') `label' leg(off);
		restore;
	};
};

gr combine p10 p50 p500 p5000 pLarge10 pLarge50 pLarge500 pLarge5000, rows(2) cols(4) `graphr'; 
gr export histBootMeansExp_byRep.pdf, replace;

* bootstrap diff;
assessBias var r(mean) "sum varExp" `data' `output'; 
use assessBiasResults, clear; 

gr tw 
	(connected bias B if(smallOrLarge==1), sort lc(red) mc(red))
	(connected bias B if(smallOrLarge==2), sort lc(blue) mc(blue)),
`graphr' leg(order(1 "Small" 2 "Large")); 
gr export connectedBootBiasMeansExp_byRep.pdf, replace; 

*=================================================================================================;
* Problem set -- selected solutions, code snippets
*=================================================================================================;
*=================================================================================================;
* ttest vs ranksum
program drop _all
program ttestVsRanksum, rclass
	syntax, [ obs(integer 1) mean1(real 0) mean2(real 0) sd1(real 1) sd2(real 1) ]
	set obs `obs'
	tempname var rdmCat cat
	g `rdmCat' = runiform()
	g `cat' = 0 if(`rdmCat'<=.4)
	replace `cat' = 1 if(`rdmCat'>.4)
	g `var' = rnormal(`mean1',`sd1') if(`cat'==0)
	replace `var' = rnormal(`mean2',`sd2') if(`cat'==1)
	ttest `var', by(`cat')
	return scalar t = r(t)
	return scalar pttest = r(p)
	ranksum `var', by(`cat');
	return scalar z = r(z);
	return scalar pranksum = 2*normprob(-abs(`r(z)'));
end;

simulate `pttest' `z' `pranksum', seed(010101) reps(1000): 
ttestVsRanksum, obs(15) mean1(0) mean2(5) sd1(1) sd2(10); 

kdensity t, n(1000) gen(t_x simulatedDensity) nograph;
g double asymptoticDensity = tden(13,t_x);
gr tw (line simulatedDensity t_x) (line asymptoticDensity t_x);

kdensity z, n(1000) gen(z_x simulatedDensityZ) nograph;
g double asymptoticDensityZ = normalden(0,z_x);	
gr tw (line simulatedDensityZ z_x) (line asymptoticDensityZ z_x);
	
*=====================================================;
* regress vs censlope;
program drop _all;
program regressVsCenslope, rclass;
	syntax [, numObs(integer 1) beta(real 2)];
	drop _all;
	set obs `numObs';
	g double var = rchi2(1);
	g y = 1 + `beta'*var + rchi2(1) - 1;
	reg y var;
	mat coeff = r(table);
	return scalar bReg = coeff[1,1];
 	return scalar se = coeff[2,1];
 	censlope y var;
	mat slope = r(cimat);
	return scalar bSlope = slope[1,2];
	return scalar lbSlope = slope[1,3];
	return scalar ubSlope = slope[1,4];
end;

* very slow because censlope runs veeeery slow;
simulate bReg = r(bReg) bSlope = r(bSlope), reps(50) seed(01010): regressVsCenslope, numObs(100);	

gr tw (kdensity bReg, lc(red)) (kdensity bSlope, lc(blue));

set seed 01010;
tempname simNumObs;
tempfile simFile;
postfile `simNumObs' numObs meanbReg meanbSlope sdReg sdSlope using `simFile', replace;
foreach n in 10 20 50 100 200 500 1000 5000 {;
	di "number of obs: `n'";
	simulate bReg = r(bReg) bSlope = r(bSlope), reps(50) seed(01010): regressVsCenslope, numObs(`n');
	sum bReg;
	local meanReg = r(mean);
	local sdReg = r(sd);
	sum bSlope;
	local meanSlope = r(mean);
	local sdSlope = r(sd);
	post `simNumObs' (`n') (`meanReg') (`sdReg') (`meanSlope') (`sdSlope');
};

postclose `simNumObs';

use simFile, clear; 
browse;

*=====================================================;
* bootstrap ranksum z-statistic, t-test t-statistic and evaluate;
use fakeData, clear;

* write program to plot distribution of bootstrapped statistics; 
program drop _all;
program plotHistAndQnorm;
	preserve;
	use boot, clear;
	sum theta;
	local mean = r(mean);
	local sd = r(sd);
	hist theta, graphr(fc(white) lc(white) ifc(white) ilc(white)) col(black) 
	xti(theta_hat) name(hist, replace) 
	plot(function normal = normalden(x,`mean',`sd'), ra(theta) lc(red)) leg(off)
	xline(`mean', lc(gray)) ti(Distribution of theta_hat, col(black));
	qnorm theta, graphr(fc(white) lc(white) ifc(white) ilc(white)) name(qnorm, replace) yti(theta_hat, col(black)) ysc(alt) mc(black) lc(red);
	gr combine hist qnorm, `graphr'; 
	restore;
end;

bootstrap theta = r(sum_obs), seed(010101) reps(1000) saving(boot, replace): ranksum var, by(cat);
plotHistAndQnorm;

* Define program to run Wilcoxon rank sum and t-test and extract statistics; 
program drop _all;
program notSimilarShape, rclass;
	syntax, [ obs(integer 1) ];
	set obs `obs';
	tempname var rdmCat cat;
	g `rdmCat' = runiform();
	g `cat' = 0 if(`rdmCat'<=.4);
	replace `cat' = 1 if(`rdmCat'>.4);
	g `var' = rnormal() if(`cat'==0);
	replace `var' = rchi2(`obs') if(`cat'==1);
	ranksum `var', by(`cat');
	return scalar z = r(z);
	ttest `var', by(`cat');
	return scalar t = r(t);
end;

* Write simulation of s=1000;
simulate z = r(z) t = r(t), reps(1000) seed(010101) saving(sim, replace): notSimilarShape, obs(50);

hist z, ti(z-statistic/Wilcoxon) name(plot1,replace);
hist t, ti(t-statistic/t-test) name(plot2, replace);

*=====================================================;
* compare performance of OLS vs median slope estimator;
* regress vs censlope;
program drop _all;
program regressVsCenslope, rclass;
	syntax [, numObs(integer 1) beta(real 2)];
	drop _all;
	set obs `numObs';
	g double var = rchi2(1);
	g y = `beta'*var + rchi2(1);
	reg y var;
	return scalar bReg = _b[var];
	return scalar se = _se[var];
	censlope y var;
	mat slope = r(cimat);
	return scalar bSlope = slope[1,2];
	return scalar lbSlope = slope[1,3];
	return scalar ubSlope = slope[1,4];	
end;

* simulate coefficent estimates of regression and median slope estimator;
simulate bReg = r(bReg) bSlope = r(bSlope), reps(100) seed(01010): 
regressVsCenslope, numObs(100) beta(2);

hist bReg, ti(OLS, col(black))  name(plot1, replace);
hist bSlope, ti(Median slope, col(black))  name(plot2, replace);
gr combine plot1 plot2;
gr export regressVsSlopeN100.pdf, replace;

* Write program that runs ttest, ranksum test, extract power;
program drop _all;
program ttestVsRanksum, rclass;
	syntax, [ obs(integer 1) alpha(real .05) hnull(real 0) ha(real 0)];
	set obs `obs';
	tempname var rdmCat cat;
	g `rdmCat' = runiform();
	g `cat' = 0 if(`rdmCat'<=.4);
	replace `cat' = 1 if(`rdmCat'>.4);
	g `var' = rnormal(`hnull') if(`cat'==0);
	replace `var' = rnormal(`ha') if(`cat'==1);
	ttest `var', by(`cat');
	return scalar powerTtest = (r(p)<`alpha');
	ranksum `var', by(`cat');
	return scalar powerRanksum = (2*normprob(-abs(`r(z)')))<`alpha';
end;

simulate pT = r(powerTtest) pR = r(powerRanksum), seed(010110) reps(1000): 
ttestVsRanksum, obs(100) hnull(0) ha(1);

*=================================================================================================;
* Exit
*=================================================================================================;
exit;
*=================================================================================================;
*=================================================================================================;
